@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -76,37 +76,28 @@
  * @return         NULL in the case of failure otherwise mountpoint
  */
 char *device_mountpoint_sysdep(Info_T inf, char *blockdev) {
-
+  FILE *mntfd;
   struct mntent *mnt;
-  FILE          *mntfd;
 
   ASSERT(inf);
   ASSERT(blockdev);
 
-
-  if((mntfd= setmntent("/etc/mtab", "r")) == NULL) {
+  if ((mntfd = setmntent("/etc/mtab", "r")) == NULL) {
     LogError("%s: Cannot open /etc/mtab file\n", prog);
     return NULL;
   }
-
-  /* First match is significant */
-  while((mnt= getmntent(mntfd)) != NULL) {
-
-    if(IS(blockdev, mnt->mnt_fsname)) {
-
+  while ((mnt = getmntent(mntfd)) != NULL) {
+    char realpathbuf[PATH_MAX+1];
+    /* Try to compare the the filesystem as is, if failed, try to use the symbolic link target */
+    if (IS(blockdev, mnt->mnt_fsname) || (realpath(mnt->mnt_fsname, realpathbuf) && ! strcasecmp(blockdev, realpathbuf))) {
       endmntent(mntfd);
-      inf->mntpath[sizeof(inf->mntpath) - 1] = 0;
-      return strncpy(inf->mntpath, mnt->mnt_dir, sizeof(inf->mntpath) - 1);
-
+      inf->priv.filesystem.mntpath = xstrdup(mnt->mnt_dir);
+      return inf->priv.filesystem.mntpath;
     }
-
   }
-
   endmntent(mntfd);
-
   LogError("Device %s not found in /etc/mtab\n", blockdev);
   return NULL;
-
 }
 
 
@@ -118,26 +109,21 @@ char *device_mountpoint_sysdep(Info_T inf, char *blockdev) {
  * @return        TRUE if informations were succesfully read otherwise FALSE
  */
 int filesystem_usage_sysdep(Info_T inf) {
-
   struct statvfs usage;
 
   ASSERT(inf);
 
-  if(statvfs(inf->mntpath, &usage) != 0) {
-    LogError("%s: Error getting usage statistics for filesystem '%s' -- %s\n",
-        prog, inf->mntpath, STRERROR);
+  if (statvfs(inf->priv.filesystem.mntpath, &usage) != 0) {
+    LogError("%s: Error getting usage statistics for filesystem '%s' -- %s\n", prog, inf->priv.filesystem.mntpath, STRERROR);
     return FALSE;
   }
-
-  inf->f_bsize=           usage.f_bsize;
-  inf->f_blocks=          usage.f_blocks;
-  inf->f_blocksfree=      usage.f_bavail;
-  inf->f_blocksfreetotal= usage.f_bfree;
-  inf->f_files=           usage.f_files;
-  inf->f_filesfree=       usage.f_ffree;
-  inf->flags=             usage.f_flag;
-
+  inf->priv.filesystem.f_bsize =           usage.f_bsize;
+  inf->priv.filesystem.f_blocks =          usage.f_blocks;
+  inf->priv.filesystem.f_blocksfree =      usage.f_bavail;
+  inf->priv.filesystem.f_blocksfreetotal = usage.f_bfree;
+  inf->priv.filesystem.f_files =           usage.f_files;
+  inf->priv.filesystem.f_filesfree =       usage.f_ffree;
+  inf->priv.filesystem.flags =             usage.f_flag;
   return TRUE;
-
 }
 
