@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -150,23 +150,28 @@ void init_env() {
  */
 static void set_sandbox(void) {
 
-  int    i;
+  int    i = 0;
   struct stat st;
   extern char **environ;
   char   *path = "PATH=/bin:/usr/bin:/sbin:/usr/sbin";
+  char   *tz;
 
   /*
-   * Purge the environment. Then make sure PATH is set; some shells default
-   * to a path with '.' first. You may have to putenv() other stuff, too,
-   * but be careful with importing too much.
+   * Purge the environment, but keep the TZ variable as the time.h family depends on it at least on AIX
    */
-  environ[0]= 0;
+  for (tz = environ[0]; tz; tz = environ[++i]) {
+    if (! strncasecmp(tz, "TZ=", 3)) {
+      environ[0] = tz;
+      environ[1] = 0;
+      break;
+    }
+  }
+  if (! tz)
+    environ[0] = 0;
   
-  if(putenv(path)) {
-    
+  if (putenv(path)) {
     LogError("%s: cannot set the PATH variable -- %s\n", prog, STRERROR);
     exit(1);
-    
   }
 
   /*
