@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -407,11 +407,10 @@ const char *Event_get_description(Event_T E) {
  * @return An action id
  */
 short Event_get_action(Event_T E) {
-  Action_T A;
+  Action_T A = NULL;
 
   ASSERT(E);
 
-  A = E->state ? E->action->failed : E->action->succeeded;
   switch (E->state) {
     case STATE_SUCCEEDED:
     case STATE_CHANGEDNOT:
