@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -124,9 +124,15 @@ Service_T servicelist_conf;   /**< The service list in conf file (c. in p.y) */
 ServiceGroup_T servicegrouplist;/**< The service group list (created in p.y) */
 SystemInfo_T systeminfo;                              /**< System infomation */
 
-pthread_t       heartbeatThread;               /**< M/Monit heartbeat thread */
-pthread_cond_t  heartbeatCond;                /**< Hearbeat wakeup condition */
-pthread_mutex_t heartbeatMutex;                          /**< Hearbeat mutex */
+pthread_t           heartbeatThread;           /**< M/Monit heartbeat thread */
+pthread_cond_t      heartbeatCond;            /**< Hearbeat wakeup condition */
+pthread_mutex_t     heartbeatMutex;                      /**< Hearbeat mutex */
+static volatile int heartbeatRunning = FALSE;     /**< Heartbeat thread flag */
+
+int ptreesize = 0;
+int oldptreesize = 0;
+ProcessTree_T *ptree = NULL;
+ProcessTree_T *oldptree = NULL;
 
 char actionnames[][STRLEN]   = {"ignore", "alert", "restart", "stop", "exec", "unmonitor", "start", "monitor", ""};
 char modenames[][STRLEN]     = {"active", "passive", "manual"};
@@ -227,6 +233,11 @@ static void do_init() {
    */
   signal(SIGPIPE, SIG_IGN);
 
+  /*
+   * Initialize the random number generator
+   */
+  srandom(time(NULL) + getpid());
+
   /*
    * Initialize the Runtime mutex. This mutex
    * is used to synchronize handling of global
@@ -320,11 +331,12 @@ static void do_reinit() {
   LogInfo("Awakened by the SIGHUP signal\n");
   LogInfo("Reinitializing %s - Control file '%s'\n", prog, Run.controlfile);
   
-  if(Run.mmonits) {
+  if(Run.mmonits && heartbeatRunning) {
     if ((status = pthread_cond_signal(&heartbeatCond)) != 0)
       LogError("%s: Failed to signal the heartbeat thread -- %s\n", prog, strerror(status));
     if ((status = pthread_join(heartbeatThread, NULL)) != 0)
       LogError("%s: Failed to stop the heartbeat thread -- %s\n", prog, strerror(status));
+    heartbeatRunning = FALSE;
   }
 
   Run.doreload = FALSE;
@@ -378,6 +390,8 @@ static void do_reinit() {
 
   if(Run.mmonits && ((status = pthread_create(&heartbeatThread, NULL, heartbeat, NULL)) != 0))
     LogError("%s: Failed to create the heartbeat thread -- %s\n", prog, strerror(status));
+  else
+    heartbeatRunning = TRUE;
 }
 
 
@@ -440,6 +454,12 @@ static void do_action(char **args) {
     status(LEVEL_NAME_FULL);
   } else if (IS(action, "summary")) {
     status(LEVEL_NAME_SUMMARY);
+  } else if (IS(action, "procmatch")) {
+    if (! service) {
+      printf("Invalid syntax - usage: procmatch \"<pattern>\"\n");
+      exit(1);
+    }
+    process_testmatch(service);
   } else if (IS(action, "quit")) {
     kill_daemon(SIGTERM);
   } else if (IS(action, "validate")) {
@@ -465,11 +485,12 @@ static void do_exit() {
     if (can_http())
       monit_http(STOP_HTTP);
 
-    if(Run.mmonits) {
+    if(Run.mmonits && heartbeatRunning) {
       if ((status = pthread_cond_signal(&heartbeatCond)) != 0)
         LogError("%s: Failed to signal the heartbeat thread -- %s\n", prog, strerror(status));
       if ((status = pthread_join(heartbeatThread, NULL)) != 0)
         LogError("%s: Failed to stop the heartbeat thread -- %s\n", prog, strerror(status));
+      heartbeatRunning = FALSE;
     }
 
     LogInfo("%s daemon with pid [%d] killed\n", prog, (int)getpid());
@@ -539,6 +560,8 @@ static void do_default() {
 
     if(Run.mmonits && ((status = pthread_create(&heartbeatThread, NULL, heartbeat, NULL)) != 0))
       LogError("%s: Failed to create the heartbeat thread -- %s\n", prog, strerror(status));
+    else
+      heartbeatRunning = TRUE;
 
     while (TRUE) {
       validate();
@@ -685,21 +708,22 @@ static void help() {
   printf(" -V            Print version number and patchlevel\n");
   printf(" -h            Print this text\n");
   printf("Optional action arguments for non-daemon mode are as follows:\n");
-  printf(" start all      - Start all services\n");
-  printf(" start name     - Only start the named service\n");
-  printf(" stop all       - Stop all services\n");
-  printf(" stop name      - Only stop the named service\n");
-  printf(" restart all    - Stop and start all services\n");
-  printf(" restart name   - Only restart the named service\n");
-  printf(" monitor all    - Enable monitoring of all services\n");
-  printf(" monitor name   - Only enable monitoring of the named service\n");
-  printf(" unmonitor all  - Disable monitoring of all services\n");
-  printf(" unmonitor name - Only disable monitoring of the named service\n");
-  printf(" reload         - Reinitialize monit\n");
-  printf(" status         - Print full status information for each service\n");
-  printf(" summary        - Print short status information for each service\n");
-  printf(" quit           - Kill monit daemon process\n");
-  printf(" validate       - Check all services and start if not running\n");
+  printf(" start all           - Start all services\n");
+  printf(" start name          - Only start the named service\n");
+  printf(" stop all            - Stop all services\n");
+  printf(" stop name           - Only stop the named service\n");
+  printf(" restart all         - Stop and start all services\n");
+  printf(" restart name        - Only restart the named service\n");
+  printf(" monitor all         - Enable monitoring of all services\n");
+  printf(" monitor name        - Only enable monitoring of the named service\n");
+  printf(" unmonitor all       - Disable monitoring of all services\n");
+  printf(" unmonitor name      - Only disable monitoring of the named service\n");
+  printf(" reload              - Reinitialize monit\n");
+  printf(" status              - Print full status information for each service\n");
+  printf(" summary             - Print short status information for each service\n");
+  printf(" quit                - Kill monit daemon process\n");
+  printf(" validate            - Check all services and start if not running\n");
+  printf(" procmatch <pattern> - Test process matching pattern\n");
   printf("\n");
   printf("(Action arguments operate on services defined in the control file)\n");
 }
@@ -709,7 +733,7 @@ static void help() {
  */
 static void version() {
   printf("This is Monit version " VERSION "\n");
-  printf("Copyright (C) 2000-2010 by Tildeslash Ltd.");
+  printf("Copyright (C) 2000-2011 Tildeslash Ltd.");
   printf(" All Rights Reserved.\n");
 }
 
@@ -718,7 +742,6 @@ static void version() {
  * M/Monit heartbeat thread
  */
 static void *heartbeat(void *args) {
-  int status;
   sigset_t ns;
   struct timespec wait;
 
@@ -727,7 +750,7 @@ static void *heartbeat(void *args) {
   LOCK(heartbeatMutex)
   {
     while (! Run.stopped && ! Run.doreload) {
-      if ((status = handle_mmonit(NULL)) == HANDLER_SUCCEEDED)
+      if (handle_mmonit(NULL) == HANDLER_SUCCEEDED)
         wait.tv_sec = time(NULL) + Run.polltime;
       else
         wait.tv_sec = time(NULL) + 1;
