@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2000-2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -97,8 +97,8 @@
 #define QUEUEMASK          0077
 /* Set file mode: "drw-------" */
 #define PRIVATEMASK        0177
-/* Set log file mode: "-rw-rw-r--" */
-#define LOGMASK            0112
+/* Set log file mode: "-rw-r-----" */
+#define LOGMASK            0137
 /* Set pid file mode: "-rw-r--r--" */
 #define MYPIDMASK          0122
 #define MYPIDDIR           PIDDIR
@@ -466,7 +466,6 @@ typedef struct myport {
   char *request_hostheader;            /**< The optional Host: header to use */
   int  request_hashtype;  /**< The optional type of hash for a req. document */
   char *pathname;                   /**< Pathname, in case of an UNIX socket */
-  char *address;               /**< Human readable destination of the socket */
   int maxforward;            /**< Optional max forward for protocol checking */
   Generic_T generic;                                /**< Generic test handle */
   int timeout;   /**< The timeout in seconds to wait for connect or read i/o */
@@ -656,49 +655,53 @@ typedef struct myfilesystem {
 
 /** Defines service data */
 typedef struct myinfo {
-
   /* Shared */
   mode_t  st_mode;                                           /**< Permission */
   uid_t   st_uid;                                           /**< Owner's uid */
   gid_t   st_gid;                                           /**< Owner's gid */
-  ino_t   st_ino;                                                 /**< Inode */
   time_t  timestamp;                                          /**< Timestamp */
 
-  /* Filesystem specific */
-  long   f_bsize;                                   /**< Transfer block size */
-  long   f_blocks;                      /**< Total data blocks in filesystem */
-  long   f_blocksfree;           /**< Free blocks available to non-superuser */
-  long   f_blocksfreetotal;                   /**< Free blocks in filesystem */
-  long   f_files;                        /**< Total file nodes in filesystem */
-  long   f_filesfree;                     /**< Free file nodes in filesystem */
-  char   mntpath[STRLEN];      /**< Filesystem file, directory or mountpoint */
-  int    inode_percent;                      /**< Used inode percentage * 10 */
-  long   inode_total;                          /**< Used inode total objects */
-  int    space_percent;                      /**< Used space percentage * 10 */
-  long   space_total;                           /**< Used space total blocks */
-  int    _flags;                       /**< Filesystem flags from last cycle */
-  int    flags;                      /**< Filesystem flags from actual cycle */
+  union {
+    struct {
+      long   f_bsize;                               /**< Transfer block size */
+      long   f_blocks;                  /**< Total data blocks in filesystem */
+      long   f_blocksfree;       /**< Free blocks available to non-superuser */
+      long   f_blocksfreetotal;               /**< Free blocks in filesystem */
+      long   f_files;                    /**< Total file nodes in filesystem */
+      long   f_filesfree;                 /**< Free file nodes in filesystem */
+      char  *mntpath;          /**< Filesystem file, directory or mountpoint */
+      int    inode_percent;                  /**< Used inode percentage * 10 */
+      long   inode_total;                      /**< Used inode total objects */
+      int    space_percent;                  /**< Used space percentage * 10 */
+      long   space_total;                       /**< Used space total blocks */
+      int    _flags;                   /**< Filesystem flags from last cycle */
+      int    flags;                  /**< Filesystem flags from actual cycle */
+    } filesystem;
 
-  /* File specific */
-  off_t st_size;                                                   /**< Size */
-  off_t readpos;                            /**< Position for regex matching */
-  ino_t st_ino_prev;                  /**< Previous inode for regex matching */
-  MD_T  cs_sum;                                                /**< Checksum */
+    struct {
+      off_t st_size;                                               /**< Size */
+      off_t readpos;                        /**< Position for regex matching */
+      ino_t st_ino;                                               /**< Inode */
+      ino_t st_ino_prev;              /**< Previous inode for regex matching */
+      MD_T  cs_sum;                                            /**< Checksum */
+    } file;
 
-  /* Process specific */
-  int    _pid;                              /**< Process PID from last cycle */
-  int    _ppid;                      /**< Process parent PID from last cycle */
-  int    pid;                             /**< Process PID from actual cycle */
-  int    ppid;                     /**< Process parent PID from actual cycle */
-  int    status_flag;
-  int    children;
-  long   mem_kbyte;    
-  long   total_mem_kbyte;
-  int    mem_percent;                                   /**< percentage * 10 */
-  int    total_mem_percent;                             /**< percentage * 10 */
-  int    cpu_percent;                                   /**< percentage * 10 */
-  int    total_cpu_percent;                             /**< percentage * 10 */
-  time_t uptime;                                         /**< Process uptime */
+    struct {
+      int    _pid;                          /**< Process PID from last cycle */
+      int    _ppid;                  /**< Process parent PID from last cycle */
+      int    pid;                         /**< Process PID from actual cycle */
+      int    ppid;                 /**< Process parent PID from actual cycle */
+      int    status_flag;
+      int    children;
+      long   mem_kbyte;    
+      long   total_mem_kbyte;
+      int    mem_percent;                               /**< percentage * 10 */
+      int    total_mem_percent;                         /**< percentage * 10 */
+      int    cpu_percent;                               /**< percentage * 10 */
+      int    total_cpu_percent;                         /**< percentage * 10 */
+      time_t uptime;                                     /**< Process uptime */
+    } process;
+  } priv;
 } *Info_T;
 
 
@@ -760,7 +763,7 @@ typedef struct myservice {
   Info_T             inf;                          /**< Service check result */
   struct timeval     collected;                /**< When were data collected */
   int                doaction;          /**< Action scheduled by http thread */
-  char               token[STRLEN];                        /**< Action token */
+  char              *token;                                /**< Action token */
 
   /** Events */
   struct myevent {
@@ -966,7 +969,7 @@ int  sha_md5_stream (FILE *, void *, void *);
 void reset_procinfo(Service_T);
 int  check_service_status(Service_T);
 void printhash(char *);  
-char *status_xml(Event_T, short, int);
+char *status_xml(Event_T, short, int, const char *);
 int  handle_mmonit(Event_T);
 int  do_wakeupcall();
 
