@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -89,7 +89,7 @@ int check_ntp3(Socket_T s)
     (NTP_MODE_CLIENT);
 
   /* Send request to NTP server */
-  if( (br= socket_write(s, ntpRequest, NTPLEN)) <= 0 ) {
+  if(socket_write(s, ntpRequest, NTPLEN) <= 0 ) {
     LogError("NTP: error sending NTP request -- %s\n", STRERROR);
     return FALSE;
   }
