@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  * Copyright (C) 2009 Alan DeKok <aland@freeradius.org> All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
@@ -179,12 +179,9 @@ int check_radius(Socket_T s) {
   secret = (P->request ? P->request : "testing123");
   secret_len = strlen(secret);
 
-  /* initialization of random param */
-  srand(time(NULL) + getpid());
-
   /* Get 16 bytes of (very bad) random data */
   for (i = 0; i < 16; i++)
-    request[i + 4] = ((unsigned int)rand()) & 0xff;
+    request[i + 4] = ((unsigned int)random()) & 0xff;
 
   /* sign the packet */
   hmac_md5(request, sizeof(request), (unsigned char *)secret, secret_len, request + 22);
