@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,6 +64,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
@@ -106,7 +110,6 @@ int check_sip(Socket_T s) {
   const char *myip;
   char *rport= "";
   char *proto;
-  int seed;
 
   ASSERT(s);
 
@@ -122,13 +125,11 @@ int check_sip(Socket_T s) {
     {
       transport="UDP";
       rport=";rport";
-      seed=1;
       break;
     }
     case SOCK_STREAM:
     {
       transport="TCP";
-      seed=2;
       break;
     }
     default:
@@ -140,9 +141,6 @@ int check_sip(Socket_T s) {
 
   myip= socket_get_local_host(s);
 
-  /* initialization of random param */
-  srand(time(NULL)+getpid()+seed);
-
   if(socket_print(s,
     "OPTIONS %s:%s SIP/2.0\r\n"
     "Via: SIP/2.0/%s %s:%d;branch=z9hG4bKh%u%s\r\n"
@@ -160,15 +158,15 @@ int check_sip(Socket_T s) {
     transport,        // via transport udp|tcp
     myip,             // who its from
     port,             // our port
-    rand(),           // branch
+    random(),         // branch
     rport,            // rport option
     P->maxforward,    // maximum forwards
     proto,            // protocol
     request,          // to
     proto,            // protocol
     myip,             // from host
-    rand(),           // tag
-    rand(),           // call id
+    random(),         // tag
+    random(),         // call id
     proto,            // protocol
     myip,             // contact host
     port,             // contact port
