@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -124,7 +124,7 @@ int sendmail(Mail_T mail) {
   ASSERT(mail);
   
   S.socket = NULL;
-  if((rv = sigsetjmp(S.error, TRUE))) {
+  if(sigsetjmp(S.error, TRUE)) {
     rv = FALSE;
     goto exit;
   } else {
@@ -135,9 +135,7 @@ int sendmail(Mail_T mail) {
   
   Util_getRFC822Date(NULL, now, STRLEN);
   
-  if(Run.mail_hostname || gethostname(S.localhost, sizeof(S.localhost)) < 0) {
-    snprintf(S.localhost, sizeof(S.localhost), "%s", Run.mail_hostname?Run.mail_hostname:LOCALHOST);
-  }
+  snprintf(S.localhost, sizeof(S.localhost), "%s", Run.mail_hostname ? Run.mail_hostname : Run.localhostname);
   
   do_status(&S);
 
@@ -190,7 +188,7 @@ int sendmail(Mail_T mail) {
     do_send(&S, "Mime-Version: 1.0\r\n");
     do_send(&S, "Content-Type: text/plain; charset=\"iso-8859-1\"\r\n");
     do_send(&S, "Content-Transfer-Encoding: 8bit\r\n");
-    do_send(&S, "Message-id: <%ld.%d@%s>\r\n", time(NULL), i, S.localhost);
+    do_send(&S, "Message-id: <%ld.%lu@%s>\r\n", time(NULL), random(), S.localhost);
     do_send(&S, "\r\n");
     do_send(&S, "%s\r\n", m->message);
     do_send(&S, ".\r\n");
