@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -250,10 +250,12 @@ pid_t Util_getPid(char *pidfile);
 
 
 /**
- * @return TRUE (i.e. the running pid id)  if
- * the process is running, otherwise FALSE
+ * Check whether the process is running
+ * @param s The service being checked
+ * @param refresh TRUE to refresh the global ptree (useful for procmatch if process was mangled by monit in the same cycle such as by restart action) or FALSE to use cached ptree
+ * @return The PID of the running running process or 0 if the process is not running.
  */
-int Util_isProcessRunning(Service_T s);
+int Util_isProcessRunning(Service_T s, int refresh);
 
 
 /**
@@ -495,6 +497,16 @@ char *Util_getEventratio(Action_T action, char *buf, int bufsize);
 char *Util_portTypeDescription(Port_T p);
 
 
+/**
+ * Print full port description <INET|UNIX>\[<host>:<port>[request]\][via TCP|TCPSSL|UDP]
+ * @param p A port structure
+ * @param buf Buffer
+ * @param bufsize Buffer size
+ * @return the buffer
+ */
+char *Util_portDescription(Port_T p, char *buf, int bufsize);
+
+
 /**
  * Print to string buffer
  * @param b A Buffer object
@@ -504,4 +516,13 @@ char *Util_portTypeDescription(Port_T p);
 void Util_stringbuffer(Buffer_T *b, const char *m, ...);
 
 
+/**
+ *  Returns the FQDN hostname or fallback to gethostname() output
+ *  @param buf the character array for hostname
+ *  @param len the length of buf
+ *  @return zero on success
+ */
+int Util_getfqdnhostname(char *buf, unsigned len);
+
+
 #endif
