@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Tildeslash Ltd. All rights reserved.
+ * Copyright (C) 2011 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3.
@@ -67,7 +67,7 @@
 /* -------------------------------------------------------------- Prototypes */
 
 
-static void document_head(Buffer_T *, int);
+static void document_head(Buffer_T *, int, const char *);
 static void document_foot(Buffer_T *);
 static void status_service(Service_T, Buffer_T *, short, int);
 static void status_servicegroup(ServiceGroup_T, Buffer_T *, short);
@@ -83,17 +83,18 @@ static void status_event(Event_T, Buffer_T *);
  * @param E An event object or NULL for general status
  * @param L Status information level
  * @param V Format version
+ * @param myip The client-side IP address
  * @return XML document or NULL in the case of error. The caller must free
 *  the memory.
  */
-char *status_xml(Event_T E, short L, int V) {
+char *status_xml(Event_T E, short L, int V, const char *myip) {
   Buffer_T  B;
   Service_T S;
   ServiceGroup_T SG;
 
   memset(&B, 0, sizeof(Buffer_T));
 
-  document_head(&B, V);
+  document_head(&B, V, myip);
 
   if (V == 2)
     Util_stringbuffer(&B, "<services>");
@@ -123,8 +124,9 @@ char *status_xml(Event_T E, short L, int V) {
  * Prints a document header into the given buffer.
  * @param B Buffer object
  * @param V Format version
+ * @param myip The client-side IP address
  */
-static void document_head(Buffer_T *B, int V) {
+static void document_head(Buffer_T *B, int V, const char *myip) {
   Util_stringbuffer(B,
     "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
   if (V == 2) {
@@ -164,7 +166,7 @@ static void document_head(Buffer_T *B, int V) {
       "<port>%d</port>"
       "<ssl>%d</ssl>"
       "</httpd>",
-      Run.bind_addr?Run.bind_addr:"",
+      Run.bind_addr ? Run.bind_addr : myip,
       Run.httpdport,
       Run.httpdssl);
 
@@ -280,11 +282,11 @@ static void status_service(Service_T S, Buffer_T *B, short L, int V) {
       if(S->type == TYPE_FILE) {
         Util_stringbuffer(B,
   		"<size>%llu</size>",
-  		(unsigned long long) S->inf->st_size);
+  		(unsigned long long) S->inf->priv.file.st_size);
         if(S->checksum) {
           Util_stringbuffer(B,
   		  "<checksum type=\"%s\">%s</checksum>",
-  		  checksumnames[S->checksum->type], S->inf->cs_sum);
+  		  checksumnames[S->checksum->type], S->inf->priv.file.cs_sum);
         }
       }
       if(S->type == TYPE_FILESYSTEM) {
@@ -295,20 +297,20 @@ static void status_service(Service_T S, Buffer_T *B, short L, int V) {
   		"<usage>%.1f MB</usage>"
                 "<total>%.1f MB</total>"
   		"</block>",
-  		S->inf->flags,
-  		S->inf->space_percent/10.,
-  		S->inf->f_bsize > 0 ? (float)S->inf->space_total / (float)1048576 * (float)S->inf->f_bsize : 0,
-                S->inf->f_bsize > 0 ? (float)S->inf->f_blocks / (float)1048576 * (float)S->inf->f_bsize : 0);
-        if(S->inf->f_files > 0) {
+  		S->inf->priv.filesystem.flags,
+  		S->inf->priv.filesystem.space_percent/10.,
+  		S->inf->priv.filesystem.f_bsize > 0 ? (float)S->inf->priv.filesystem.space_total / (float)1048576 * (float)S->inf->priv.filesystem.f_bsize : 0,
+                S->inf->priv.filesystem.f_bsize > 0 ? (float)S->inf->priv.filesystem.f_blocks / (float)1048576 * (float)S->inf->priv.filesystem.f_bsize : 0);
+        if(S->inf->priv.filesystem.f_files > 0) {
           Util_stringbuffer(B,
   		  "<inode>"
                   "<percent>%.1f</percent>"
                   "<usage>%ld</usage>"
                   "<total>%ld</total>"
 		  "</inode>",
-		  S->inf->inode_percent/10.,
-		  S->inf->inode_total,
-                  S->inf->f_files);
+		  S->inf->priv.filesystem.inode_percent/10.,
+		  S->inf->priv.filesystem.inode_total,
+                  S->inf->priv.filesystem.f_files);
         }
       }
       if(S->type == TYPE_PROCESS) {
@@ -316,9 +318,9 @@ static void status_service(Service_T S, Buffer_T *B, short L, int V) {
   		"<pid>%d</pid>"
   		"<ppid>%d</ppid>"
   		"<uptime>%ld</uptime>",
-  		S->inf->pid,
-  		S->inf->ppid,
-  		(long)S->inf->uptime);
+  		S->inf->priv.process.pid,
+  		S->inf->priv.process.ppid,
+  		(long)S->inf->priv.process.uptime);
         if(Run.doprocess) {
           Util_stringbuffer(B,
   		  "<children>%d</children>"
@@ -332,13 +334,13 @@ static void status_service(Service_T S, Buffer_T *B, short L, int V) {
 		  "<percent>%.1f</percent>"
 		  "<percenttotal>%.1f</percenttotal>"
 		  "</cpu>",
-		  S->inf->children,
-		  S->inf->mem_percent/10.0,
-	  	  S->inf->total_mem_percent/10.0,
-  		  S->inf->mem_kbyte,
-  		  S->inf->total_mem_kbyte,
-  		  S->inf->cpu_percent/10.0,
-  		  S->inf->total_cpu_percent/10.0);
+		  S->inf->priv.process.children,
+		  S->inf->priv.process.mem_percent/10.0,
+	  	  S->inf->priv.process.total_mem_percent/10.0,
+  		  S->inf->priv.process.mem_kbyte,
+  		  S->inf->priv.process.total_mem_kbyte,
+  		  S->inf->priv.process.cpu_percent/10.0,
+  		  S->inf->priv.process.total_cpu_percent/10.0);
         }
       }
       if(S->type == TYPE_HOST && S->icmplist) {
@@ -476,7 +478,7 @@ static void status_event(Event_T E, Buffer_T *B) {
     Event_get_state(E),
     Event_get_action(E),
     Event_get_message(E));
-  if ((s = Event_get_source(E)) && *s->token)
+  if ((s = Event_get_source(E)) && s->token)
     Util_stringbuffer(B, "<token>%s</token>", s->token);
   Util_stringbuffer(B,
     "</event>");
